//////////////////////////////////////////////////////////////////////////////
//
//      Copyright  1999 Zentropic Computing, All rights reserved
//
// Authors:             Trevor woolven
// Original date:       Thu 15 Jul 1999
//
// Posix Queues interface for Real Time Linux.
//
//
// The contents of this file are subject to the Mozilla Public
// License Version 1.0 (the "License"); you may not use this file
// except in compliance with the License. You may obtain a copy of
// the License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS
// IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _RTAI_PQUEUE_H
#define _RTAI_PQUEUE_H
static char id_rtai_pqueue_h[] __attribute__ ((unused)) = "@(#)$Id: rtai_pqueue.h,v 1.1 1999/09/03 10:17:45 trevw Exp $";

#include <linux/sched.h>
#include <linux/signal.h>

#include <rtai.h>
#include <rtai_sched.h>


// ----------------------------------------------------------------------------
#define TASK_FPU_DISABLE 0
#define TASK_FPU_ENABLE 1

#ifndef TRUE
#define TRUE    1
#define FALSE   0
#endif

#ifndef OK
#define OK      0
#define ERROR   -1
#endif

//Module configuration controls
#define	MAX_PQUEUES	16	//Maximum number of message queues in module
#define	MAX_MSGSIZE	50	//Maximum message size per queue (bytes)
#define	MAX_MSGS	10	//Maximum number of messages per queue

//Posix definitions
#define	MQ_OPEN_MAX	8	//Maximum number of message queues per process
#define	MQ_PRIO_MAX	32	//Maximum number of message priorities
#define	MQ_BLOCK	0	//Flag to set queue into blocking mode
#define	MQ_NONBLOCK	1	//Flag to set queue into non-blocking mode
#define MQ_NAME_MAX	80	//Maximum length of a queue name string

#define MQ_MIN_MSG_PRIORITY 0		//Lowest priority message
#define MQ_MAX_MSG_PRIROITY MQ_PRIO_MAX //Highest priority message

struct mq_attr {
    long mq_maxmsg;		//Maximum number of messages in queue
    long mq_msgsize;		//Maximum size of a message (in bytes)
    long mq_flags;		//Behaviour specifiers
    long mq_curmsgs;		//Number of messages currently in queue
};
typedef struct mq_attr MQ_ATTR;
MQ_ATTR DEFAULT_PQUEUE_ATTRS = {MAX_MSGS, MAX_MSGSIZE, MQ_NONBLOCK, 0};
MQ_ATTR EMPTY_PQUEUE_ATTRS  = {0,0,0,0};

struct msg_hdr {
    uint in_use;		//Is this message slot in queue being used? 
    size_t size;		//Actual message size
    uint priority;		//Message priority
    struct msg *next;		//Pointer to next message in queue
};
typedef struct msg_hdr MSG_HDR;

struct msg {
    MSG_HDR hdr;
    char data;			//Anchor point for message data
};
typedef struct msg MSG;
 
struct _pqueue_descr_struct {
    RT_TASK *owner;		//Task that created the queue
    int open_count;		//Count of the number of tasks that have
				// 'opened' the queue for access
    char *q_name;		//Name supplied for queue
    uint q_id;			//Queue Id (index into static list of queues)
    MSG *base;			//Pointer to the base of the queue
    MSG *head;			//Pointer to element at front of queue
    MSG *tail;			//Pointer to element at back of queue
    int oflags;			//Queue access permissions & blocking spec
    mode_t permissions;		//Permissions granted by creator (ugo, rwx)
    struct mq_attr attrs;	//Queue attributes
};
typedef struct _pqueue_descr_struct MSG_QUEUE;

typedef unsigned long mqd_t;
#define	INVALID_PQUEUE	-1

typedef enum {FOR_READ, FOR_WRITE} Q_ACCESS;

// ----------------------------------------------------------------------------

//Create or open a message queue
extern mqd_t mq_open(char *mq_name, int oflags, mode_t permissions, 
						struct mq_attr *mq_attr);

//Receive a message from a message queue
extern size_t mq_receive(mqd_t mq, char *msg_buffer, 
				size_t buflen, unsigned int *msgprio);

//Send a message to a queue
extern int mq_send(mqd_t mq, const char *msg, size_t msglen, 
						unsigned int msgprio);

//Close a message queue (note that the queue remains in existance!)
extern int mq_close(mqd_t mq);

//Get the attributes of a message queue
extern int mq_getattr(mqd_t mq, struct mq_attr *attrbuf);

//Set a subset of a message queue's attributes
extern int mq_setattr(mqd_t mq, const struct mq_attr *new_attrs,
				struct mq_attr *old_attrs);

//Register a request to be notified whenever a message arrives on an empty
// queue
extern int mq_notify(mqd_t mq, const struct sigevent *notification);

//Destroy a message queue
extern int mq_unlink(mqd_t mq);

// ---------------------------------< eof >------------------------------------
#endif  // _RTAI_PQUEUE_H_
